@@ -36,9 +36,11 @@ import mimeparse
 import six
 from six.moves import http_cookies
 
+from falcon import DEFAULT_MEDIA_TYPE
 from falcon import errors
 from falcon import request_helpers as helpers
 from falcon import util
+from falcon.media import Handlers
 from falcon.util.uri import parse_host, parse_query_string, unquote_string
 
 # NOTE(tbug): In some cases, http_cookies is not a module
@@ -72,12 +74,76 @@ class Request(object):
         options (dict): Set of global options passed from the API handler.
 
     Attributes:
-        scheme (str): Either 'http' or 'https'.
+        env (dict): Reference to the WSGI environ ``dict`` passed in from the
+            server. (See also PEP-3333.)
+        context (dict): Dictionary to hold any data about the request which is
+            specific to your app (e.g. session object). Falcon itself will
+            not interact with this attribute after it has been initialized.
+        context_type (class): Class variable that determines the factory or
+            type to use for initializing the `context` attribute. By default,
+            the framework will instantiate standard ``dict`` objects. However,
+            you may override this behavior by creating a custom child class of
+            ``falcon.Request``, and then passing that new class to
+            `falcon.API()` by way of the latter's `request_type` parameter.
+
+            Note:
+                When overriding `context_type` with a factory function (as
+                opposed to a class), the function is called like a method of
+                the current Request instance. Therefore the first argument is
+                the Request instance itself (self).
+        scheme (str): URL scheme used for the request. Either 'http' or
+            'https'.
+
+            Note:
+                If the request was proxied, the scheme may not
+                match what was originally requested by the client.
+                :py:attr:`forwarded_scheme` can be used, instead,
+                to handle such cases.
+
+        forwarded_scheme (str): Original URL scheme requested by the
+            user agent, if the request was proxied. Typical values are
+            'http' or 'https'.
+
+            The following request headers are checked, in order of
+            preference, to determine the forwarded scheme:
+
+                - ``Forwarded``
+                - ``X-Forwarded-For``
+
+            If none of these headers are available, or if the
+            Forwarded header is available but does not contain a
+            "proto" parameter in the first hop, the value of
+            :attr:`scheme` is returned instead.
+
+            (See also: RFC 7239, Section 1)
+
         protocol (str): Deprecated alias for `scheme`. Will be removed
             in a future release.
         method (str): HTTP method requested (e.g., 'GET', 'POST', etc.)
-        host (str): Hostname requested by the client
-        port (int): Port used for the request. If the request URL does
+        host (str): Host request header field
+        forwarded_host (str): Original host request header as received
+            by the first proxy in front of the application server.
+
+            The following request headers are checked, in order of
+            preference, to determine the forwarded scheme:
+
+                - ``Forwarded``
+                - ``X-Forwarded-Host``
+
+            If none of the above headers are available, or if the
+            Forwarded header is available but the "host"
+            parameter is not included in the first hop, the value of
+            :attr:`host` is returned instead.
+
+            Note:
+                Reverse proxies are often configured to set the Host
+                header directly to the one that was originally
+                requested by the user agent; in that case, using
+                :attr:`host` is sufficient.
+
+            (See also: RFC 7239, Section 4)
+
+        port (int): Port used for the request. If the request URI does
             not specify a port, the default one for the given schema is
             returned (80 for HTTP and 443 for HTTPS).
         netloc (str): Returns the 'host:port' portion of the request
@@ -91,10 +157,54 @@ class Request(object):
                 If the hostname in the request is an IP address, the value
                 for `subdomain` is undefined.
 
-        env (dict): Reference to the WSGI environ ``dict`` passed in from the
-            server. See also PEP-3333.
-        app (str): Name of the WSGI app (if using WSGI's notion of virtual
-            hosting).
+        app (str): The initial portion of the request URI's path that
+            corresponds to the application object, so that the
+            application knows its virtual "location". This may be an
+            empty string, if the application corresponds to the "root"
+            of the server.
+
+            (Corresponds to the "SCRIPT_NAME" environ variable defined
+            by PEP-3333.)
+        uri (str): The fully-qualified URI for the request.
+        url (str): Alias for `uri`.
+        forwarded_uri (str): Original URI for proxied requests. Uses
+            :attr:`forwarded_scheme` and :attr:`forwarded_host` in
+            order to reconstruct the original URI requested by the user
+            agent.
+        relative_uri (str): The path and query string portion of the
+            request URI, omitting the scheme and host.
+        prefix (str): The prefix of the request URI, including scheme,
+            host, and WSGI app (if any).
+        forwarded_prefix (str): The prefix of the original URI for
+            proxied requests. Uses :attr:`forwarded_scheme` and
+            :attr:`forwarded_host` in order to reconstruct the
+            original URI.
+        path (str): Path portion of the request URI (not including query
+            string).
+
+            Note:
+                `req.path` may be set to a new value by a `process_request()`
+                middleware method in order to influence routing.
+        query_string (str): Query string portion of the request URI, without
+            the preceding '?' character.
+        uri_template (str): The template for the route that was matched for
+            this request. May be ``None`` if the request has not yet been
+            routed, as would be the case for `process_request()` middleware
+            methods. May also be ``None`` if your app uses a custom routing
+            engine and the engine does not provide the URI template when
+            resolving a route.
+        remote_addr(str): IP address of the closest client or proxy to
+            the WSGI server.
+
+            This property is determined by the value of ``REMOTE_ADDR``
+            in the WSGI environment dict. Since this address is not
+            derived from an HTTP header, clients and proxies can not
+            forge it.
+
+            Note:
+                If your application is behind one or more reverse
+                proxies, you can use :py:attr:`~.access_route`
+                to retrieve the real IP address of the client.
         access_route(list): IP address of the original client, as well
             as any known addresses of proxies fronting the WSGI server.
 
@@ -121,58 +231,22 @@ class Request(object):
                 using them. Do not rely on the access route to authorize
                 requests.
 
-        remote_addr(str): IP address of the closest client or proxy to
-            the WSGI server.
+        forwarded (list): Value of the Forwarded header, as a parsed list
+            of :class:`falcon.Forwarded` objects, or ``None`` if the header
+            is missing.
 
-            This property is determined by the value of ``REMOTE_ADDR``
-            in the WSGI environment dict. Since this address is not
-            derived from an HTTP header, clients and proxies can not
-            forge it.
-
-            Note:
-                If your application is behind one or more reverse
-                proxies, you can use :py:attr:`~.access_route`
-                to retrieve the real IP address of the client.
-
-        context (dict): Dictionary to hold any data about the request which is
-            specific to your app (e.g. session object). Falcon itself will
-            not interact with this attribute after it has been initialized.
-        context_type (class): Class variable that determines the factory or
-            type to use for initializing the `context` attribute. By default,
-            the framework will instantiate standard ``dict`` objects. However,
-            you may override this behavior by creating a custom child class of
-            ``falcon.Request``, and then passing that new class to
-            `falcon.API()` by way of the latter's `request_type` parameter.
-
-            Note:
-                When overriding `context_type` with a factory function (as
-                opposed to a class), the function is called like a method of
-                the current Request instance. Therefore the first argument is
-                the Request instance itself (self).
-        uri (str): The fully-qualified URI for the request.
-        url (str): alias for `uri`.
-        relative_uri (str): The path + query string portion of the full URI.
-        path (str): Path portion of the request URL (not including query
-            string).
-
-            Note:
-                `req.path` may be set to a new value by a `process_request()`
-                middleware method in order to influence routing.
-        query_string (str): Query string portion of the request URL, without
-            the preceding '?' character.
-        uri_template (str): The template for the route that was matched for
-            this request. May be ``None`` if the request has not yet been
-            routed, as would be the case for `process_request()` middleware
-            methods. May also be ``None`` if your app uses a custom routing
-            engine and the engine does not provide the URI template when
-            resolving a route.
-
-        user_agent (str): Value of the User-Agent header, or ``None`` if the
-            header is missing.
-        accept (str): Value of the Accept header, or '*/*' if the header is
-            missing.
+            (See also: RFC 7239, Section 4)
+        date (datetime): Value of the Date header, converted to a
+            ``datetime`` instance. The header value is assumed to
+            conform to RFC 1123.
         auth (str): Value of the Authorization header, or ``None`` if the
             header is missing.
+        user_agent (str): Value of the User-Agent header, or ``None`` if the
+            header is missing.
+        referer (str): Value of the Referer header, or ``None`` if
+            the header is missing.
+        accept (str): Value of the Accept header, or '*/*' if the header is
+            missing.
         client_accepts_json (bool): ``True`` if the Accept header indicates
             that the client is willing to receive JSON, otherwise ``False``.
         client_accepts_msgpack (bool): ``True`` if the Accept header indicates
@@ -180,6 +254,9 @@ class Request(object):
             ``False``.
         client_accepts_xml (bool): ``True`` if the Accept header indicates that
             the client is willing to receive XML, otherwise ``False``.
+        cookies (dict):
+            A dict of name/value cookie pairs. (See also:
+            :ref:`Getting Cookies <getting-cookies>`)
         content_type (str): Value of the Content-Type header, or ``None`` if
             the header is missing.
         content_length (int): Value of the Content-Length header converted
@@ -249,11 +326,20 @@ class Request(object):
 
                 doc = json.load(req.bounded_stream)
 
-        date (datetime): Value of the Date header, converted to a
-            ``datetime`` instance. The header value is assumed to
-            conform to RFC 1123.
         expect (str): Value of the Expect header, or ``None`` if the
             header is missing.
+        media (object): Returns a deserialized form of the request stream.
+            When called, it will attempt to deserialize the request stream
+            using the Content-Type header as well as the media-type handlers
+            configured via :class:`falcon.RequestOptions`.
+
+            See :ref:`media` for more information regarding media handling.
+
+            Warning:
+                This operation will consume the request stream the first time
+                it's called and cache the results. Follow-up calls will just
+                retrieve a cached version of the object.
+
         range (tuple of int): A 2-member ``tuple`` parsed from the value of the
             Range header.
 
@@ -292,36 +378,37 @@ class Request(object):
             string, the value mapped to that parameter key will be a list of
             all the values in the order seen.
 
-        cookies (dict):
-            A dict of name/value cookie pairs.
-            See also: :ref:`Getting Cookies <getting-cookies>`
-
         options (dict): Set of global options passed from the API handler.
     """
 
     __slots__ = (
+        '__dict__',
+        '_bounded_stream',
+        '_cached_access_route',
+        '_cached_forwarded',
+        '_cached_forwarded_prefix',
+        '_cached_forwarded_uri',
         '_cached_headers',
-        '_cached_uri',
+        '_cached_prefix',
         '_cached_relative_uri',
+        '_cached_uri',
+        '_cookies',
+        '_params',
+        '_wsgierrors',
         'content_type',
+        'context',
         'env',
         'method',
-        '_params',
+        'options',
         'path',
         'query_string',
         'stream',
-        '_bounded_stream',
-        'context',
-        '_wsgierrors',
-        'options',
-        '_cookies',
-        '_cached_access_route',
-        '__dict__',
         'uri_template',
+        '_media',
     )
 
     # Child classes may override this
-    context_type = None
+    context_type = dict
 
     _wsgi_input_type_known = False
     _always_wrap_wsgi_input = False
@@ -334,22 +421,22 @@ class Request(object):
         self.method = env['REQUEST_METHOD']
 
         self.uri_template = None
+        self._media = None
 
-        # Normalize path
-        path = env['PATH_INFO']
-        if path:
-            if six.PY3:
-                # PEP 3333 specifies that PATH_INFO variable are always
-                # "bytes tunneled as latin-1" and must be encoded back
-                path = path.encode('latin1').decode('utf-8', 'replace')
+        # NOTE(kgriffs): PEP 3333 specifies that PATH_INFO may be the
+        # empty string, so normalize it in that case.
+        path = env['PATH_INFO'] or '/'
 
-            if (self.options.strip_url_path_trailing_slash and
-                    len(path) != 1 and path.endswith('/')):
-                self.path = path[:-1]
-            else:
-                self.path = path
+        if six.PY3:
+            # PEP 3333 specifies that PATH_INFO variable are always
+            # "bytes tunneled as latin-1" and must be encoded back
+            path = path.encode('latin1').decode('utf-8', 'replace')
+
+        if (self.options.strip_url_path_trailing_slash and
+                len(path) != 1 and path.endswith('/')):
+            self.path = path[:-1]
         else:
-            self.path = '/'
+            self.path = path
 
         # PERF(ueg1990): try/catch cheaper and faster (and more Pythonic)
         try:
@@ -370,10 +457,14 @@ class Request(object):
 
         self._cookies = None
 
-        self._cached_headers = None
-        self._cached_uri = None
-        self._cached_relative_uri = None
         self._cached_access_route = None
+        self._cached_forwarded = None
+        self._cached_forwarded_prefix = None
+        self._cached_forwarded_uri = None
+        self._cached_headers = None
+        self._cached_prefix = None
+        self._cached_relative_uri = None
+        self._cached_uri = None
 
         try:
             self.content_type = self.env['CONTENT_TYPE']
@@ -382,7 +473,12 @@ class Request(object):
 
         # NOTE(kgriffs): Wrap wsgi.input if needed to make read() more robust,
         # normalizing semantics between, e.g., gunicorn and wsgiref.
-        if not Request._wsgi_input_type_known:
+        #
+        # PERF(kgriffs): Accessing via self when reading is faster than
+        # via the class name. But we must set the variables using the
+        # class name so they are picked up by all future instantiations
+        # of the class.
+        if not self._wsgi_input_type_known:
             Request._always_wrap_wsgi_input = isinstance(
                 env['wsgi.input'],
                 (NativeStream, InputWrapper)
@@ -390,7 +486,7 @@ class Request(object):
 
             Request._wsgi_input_type_known = True
 
-        if Request._always_wrap_wsgi_input:
+        if self._always_wrap_wsgi_input:
             # TODO(kgriffs): In Falcon 2.0, stop wrapping stream since it is
             # less useful now that we have bounded_stream.
             self.stream = self._get_wrapped_wsgi_input()
@@ -415,11 +511,10 @@ class Request(object):
         ):
             self._parse_form_urlencoded()
 
-        if self.context_type is None:
-            # Literal syntax is more efficient than using dict()
-            self.context = {}
-        else:
-            self.context = self.context_type()
+        self.context = self.context_type()
+
+    def __repr__(self):
+        return '<%s: %s %r>' % (self.__class__.__name__, self.method, self.url)
 
     # ------------------------------------------------------------------------
     # Properties
@@ -434,6 +529,74 @@ class Request(object):
     if_none_match = helpers.header_property('HTTP_IF_NONE_MATCH')
     if_range = helpers.header_property('HTTP_IF_RANGE')
 
+    referer = helpers.header_property('HTTP_REFERER')
+
+    @property
+    def forwarded(self):
+        # PERF(kgriffs): We could DRY up this memoization pattern using
+        # a decorator, but that would incur additional overhead without
+        # resorting to some trickery to rewrite the body of the method
+        # itself (vs. simply wrapping it with some memoization logic).
+        # At some point we might look into this but I don't think
+        # it's worth it right now.
+        if self._cached_forwarded is None:
+            # PERF(kgriffs): If someone is calling this, they are probably
+            # confident that the header exists, so most of the time we
+            # expect this call to succeed. Therefore, we won't need to
+            # pay the penalty of a raised exception in most cases, and
+            # there is no need to spend extra cycles calling get() or
+            # checking beforehand whether the key is in the dict.
+            try:
+                forwarded = self.env['HTTP_FORWARDED']
+            except KeyError:
+                return None
+
+            parsed_elements = []
+
+            for element in forwarded.split(','):
+                parsed_element = Forwarded()
+
+                # NOTE(kgriffs): Calling strip() is necessary here since
+                # "an HTTP list allows white spaces to occur between the
+                # identifiers".
+
+                # (See also: RFC 7239, Section 7.1)
+                for param in element.strip().split(';'):
+                    # PERF(kgriffs): partition() is faster than split().
+                    name, __, value = param.partition('=')
+                    if not value:
+                        # NOTE(kgriffs): The '=' separator was not found or
+                        # the value was missing. Ignore this malformed
+                        # param.
+                        continue
+
+                    # NOTE(kgriffs): According to RFC 7239, parameter
+                    # names are case-insensitive.
+                    name = name.lower()
+                    value = unquote_string(value)
+                    if name == 'by':
+                        parsed_element.dest = value
+                    elif name == 'for':
+                        parsed_element.src = value
+                    elif name == 'host':
+                        parsed_element.host = value
+                    elif name == 'proto':
+                        # NOTE(kgriffs): RFC 7239 only requires that
+                        # the "proto" value conform to the Host ABNF
+                        # described in RFC 7230. The Host ABNF, in turn,
+                        # does not require that the scheme be in any
+                        # particular case, so we normalize it here to be
+                        # consistent with the WSGI spec that *does*
+                        # require the value of 'wsgi.url_scheme' to be
+                        # either 'http' or 'https' (case-sensitive).
+                        parsed_element.scheme = value.lower()
+
+                parsed_elements.append(parsed_element)
+
+            self._cached_forwarded = parsed_elements
+
+        return self._cached_forwarded
+
     @property
     def client_accepts_json(self):
         return self.client_accepts('application/json')
@@ -556,30 +719,56 @@ class Request(object):
 
     @property
     def app(self):
-        return self.env.get('SCRIPT_NAME', '')
+        # PERF(kgriffs): try..except is faster than get() assuming that
+        # we normally expect the key to exist. Even though PEP-3333
+        # allows WSGI servers to omit the key when the value is an
+        # empty string, uwsgi, gunicorn, waitress, and wsgiref all
+        # include it even in that case.
+        try:
+            return self.env['SCRIPT_NAME']
+        except KeyError:
+            return ''
 
     @property
     def scheme(self):
         return self.env['wsgi.url_scheme']
 
+    @property
+    def forwarded_scheme(self):
+        # PERF(kgriffs): Since the Forwarded header is still relatively
+        # new, we expect X-Forwarded-Proto to be more common, so
+        # try to avoid calling self.forwarded if we can, since it uses a
+        # try...catch that will usually result in a relatively expensive
+        # raised exception.
+        if 'HTTP_FORWARDED' in self.env:
+            first_hop = self.forwarded[0]
+            scheme = first_hop.scheme or self.scheme
+        else:
+            # PERF(kgriffs): This call should normally succeed, so
+            # just go for it without wasting time checking it
+            # first. Note also that the indexing operator is
+            # slightly faster than using get().
+            try:
+                scheme = self.env['HTTP_X_FORWARDED_PROTO'].lower()
+            except KeyError:
+                scheme = self.env['wsgi.url_scheme']
+
+        return scheme
+
     # TODO(kgriffs): Remove this deprecated alias in Falcon 2.0
     protocol = scheme
 
     @property
     def uri(self):
         if self._cached_uri is None:
-            protocol = self.env['wsgi.url_scheme']
+            scheme = self.env['wsgi.url_scheme']
 
             # PERF: For small numbers of items, '+' is faster
             # than ''.join(...). Concatenation is also generally
             # faster than formatting.
-            value = (protocol + '://' +
+            value = (scheme + '://' +
                      self.netloc +
-                     self.app +
-                     self.path)
-
-            if self.query_string:
-                value = value + '?' + self.query_string
+                     self.relative_uri)
 
             self._cached_uri = value
 
@@ -587,6 +776,53 @@ class Request(object):
 
     url = uri
 
+    @property
+    def forwarded_uri(self):
+        if self._cached_forwarded_uri is None:
+            # PERF: For small numbers of items, '+' is faster
+            # than ''.join(...). Concatenation is also generally
+            # faster than formatting.
+            value = (self.forwarded_scheme + '://' +
+                     self.forwarded_host +
+                     self.relative_uri)
+
+            self._cached_forwarded_uri = value
+
+        return self._cached_forwarded_uri
+
+    @property
+    def relative_uri(self):
+        if self._cached_relative_uri is None:
+            if self.query_string:
+                self._cached_relative_uri = (self.app + self.path + '?' +
+                                             self.query_string)
+            else:
+                self._cached_relative_uri = self.app + self.path
+
+        return self._cached_relative_uri
+
+    @property
+    def prefix(self):
+        if self._cached_prefix is None:
+            self._cached_prefix = (
+                self.env['wsgi.url_scheme'] + '://' +
+                self.netloc +
+                self.app
+            )
+
+        return self._cached_prefix
+
+    @property
+    def forwarded_prefix(self):
+        if self._cached_forwarded_prefix is None:
+            self._cached_forwarded_prefix = (
+                self.forwarded_scheme + '://' +
+                self.forwarded_host +
+                self.app
+            )
+
+        return self._cached_forwarded_prefix
+
     @property
     def host(self):
         try:
@@ -602,23 +838,34 @@ class Request(object):
 
         return host
 
+    @property
+    def forwarded_host(self):
+        # PERF(kgriffs): Since the Forwarded header is still relatively
+        # new, we expect X-Forwarded-Host to be more common, so
+        # try to avoid calling self.forwarded if we can, since it uses a
+        # try...catch that will usually result in a relatively expensive
+        # raised exception.
+        if 'HTTP_FORWARDED' in self.env:
+            first_hop = self.forwarded[0]
+            host = first_hop.host or self.host
+        else:
+            # PERF(kgriffs): This call should normally succeed, assuming
+            # that the caller is expecting a forwarded header, so
+            # just go for it without wasting time checking it
+            # first.
+            try:
+                host = self.env['HTTP_X_FORWARDED_HOST']
+            except KeyError:
+                host = self.host
+
+        return host
+
     @property
     def subdomain(self):
         # PERF(kgriffs): .partition is slightly faster than .split
         subdomain, sep, remainder = self.host.partition('.')
         return subdomain if sep else None
 
-    @property
-    def relative_uri(self):
-        if self._cached_relative_uri is None:
-            if self.query_string:
-                self._cached_relative_uri = (self.app + self.path + '?' +
-                                             self.query_string)
-            else:
-                self._cached_relative_uri = self.app + self.path
-
-        return self._cached_relative_uri
-
     @property
     def headers(self):
         # NOTE(kgriffs: First time here will cache the dict so all we
@@ -649,7 +896,13 @@ class Request(object):
             # NOTE(tbug): We might want to look into parsing
             # cookies ourselves. The SimpleCookie is doing a
             # lot if stuff only required to SEND cookies.
-            parser = SimpleCookie(self.get_header('Cookie'))
+            cookie_header = self.get_header('Cookie', default='')
+            parser = SimpleCookie()
+            for cookie_part in cookie_header.split('; '):
+                try:
+                    parser.load(cookie_part)
+                except http_cookies.CookieError:
+                    pass
             cookies = {}
             for morsel in parser.values():
                 cookies[morsel.key] = morsel.value
@@ -673,7 +926,11 @@ class Request(object):
             # aware that an upstream proxy is malfunctioning.
 
             if 'HTTP_FORWARDED' in self.env:
-                self._cached_access_route = self._parse_rfc_forwarded()
+                self._cached_access_route = []
+                for hop in self.forwarded:
+                    if hop.src is not None:
+                        host, __ = parse_host(hop.src)
+                        self._cached_access_route.append(host)
             elif 'HTTP_X_FORWARDED_FOR' in self.env:
                 addresses = self.env['HTTP_X_FORWARDED_FOR'].split(',')
                 self._cached_access_route = [ip.strip() for ip in addresses]
@@ -715,7 +972,8 @@ class Request(object):
         # NOTE(kgriffs): According to PEP-3333 we should first
         # try to use the Host header if present.
         #
-        # PERF(kgriffs): try..except is faster than .get
+        # PERF(kgriffs): try..except is faster than get() when we
+        # expect the key to be present most of the time.
         try:
             netloc_value = env['HTTP_HOST']
         except KeyError:
@@ -731,6 +989,23 @@ class Request(object):
 
         return netloc_value
 
+    @property
+    def media(self):
+        if self._media:
+            return self._media
+
+        handler = self.options.media_handlers.find_by_media_type(
+            self.content_type,
+            self.options.default_media_type
+        )
+
+        # Consume the stream
+        raw = self.bounded_stream.read()
+
+        # Deserialize and Return
+        self._media = handler.deserialize(raw)
+        return self._media
+
     # ------------------------------------------------------------------------
     # Methods
     # ------------------------------------------------------------------------
@@ -1141,6 +1416,51 @@ class Request(object):
 
         raise errors.HTTPMissingParam(name)
 
+    def get_param_as_datetime(self, name, format_string='%Y-%m-%dT%H:%M:%SZ',
+                              required=False, store=None):
+        """Return the value of a query string parameter as a datetime.
+
+        Args:
+            name (str): Parameter name, case-sensitive (e.g., 'ids').
+
+        Keyword Args:
+            format_string (str): String used to parse the param value
+                into a datetime. Any format recognized by strptime() is
+                supported (default ``'%Y-%m-%dT%H:%M:%SZ'``).
+            required (bool): Set to ``True`` to raise
+                ``HTTPBadRequest`` instead of returning ``None`` when the
+                parameter is not found (default ``False``).
+            store (dict): A ``dict``-like object in which to place
+                the value of the param, but only if the param is found (default
+                ``None``).
+        Returns:
+            datetime.datetime: The value of the param if it is found and can be
+            converted to a ``datetime`` according to the supplied format
+            string. If the param is not found, returns ``None`` unless
+            required is ``True``.
+
+        Raises:
+            HTTPBadRequest: A required param is missing from the request.
+            HTTPInvalidParam: A transform function raised an instance of
+                ``ValueError``.
+        """
+
+        param_value = self.get_param(name, required=required)
+
+        if param_value is None:
+            return None
+
+        try:
+            date_time = strptime(param_value, format_string)
+        except ValueError:
+            msg = 'The date value does not match the required format.'
+            raise errors.HTTPInvalidParam(msg, name)
+
+        if store is not None:
+            store[name] = date_time
+
+        return date_time
+
     def get_param_as_date(self, name, format_string='%Y-%m-%d',
                           required=False, store=None):
         """Return the value of a query string parameter as a date.
@@ -1170,17 +1490,12 @@ class Request(object):
                 ``ValueError``.
         """
 
-        param_value = self.get_param(name, required=required)
-
-        if param_value is None:
+        date_time = self.get_param_as_datetime(name, format_string, required)
+        if date_time:
+            date = date_time.date()
+        else:
             return None
 
-        try:
-            date = strptime(param_value, format_string).date()
-        except ValueError:
-            msg = 'The date value does not match the required format.'
-            raise errors.HTTPInvalidParam(msg, name)
-
         if store is not None:
             store[name] = date
 
@@ -1303,33 +1618,6 @@ class Request(object):
 
             self._params.update(extra_params)
 
-    def _parse_rfc_forwarded(self):
-        """Parse RFC 7239 "Forwarded" header.
-
-        Returns:
-            list: addresses derived from "for" parameters.
-        """
-
-        addr = []
-
-        for forwarded in self.env['HTTP_FORWARDED'].split(','):
-            for param in forwarded.split(';'):
-                # PERF(kgriffs): Partition() is faster than split().
-                key, _, val = param.strip().partition('=')
-                if not val:
-                    # NOTE(kgriffs): The '=' separator was not found or
-                    # it was, but the value was missing.
-                    continue
-
-                if key.lower() != 'for':
-                    # We only want "for" params
-                    continue
-
-                host, _ = parse_host(unquote_string(val))
-                addr.append(host)
-
-        return addr
-
 
 # PERF: To avoid typos and improve storage space and speed over a dict.
 class RequestOptions(object):
@@ -1387,12 +1675,25 @@ class RequestOptions(object):
             forward slash. However, this behavior can be problematic in
             certain cases, such as when working with authentication
             schemes that employ URL-based signatures.
+
+        default_media_type (str): The default media-type to use when
+            deserializing a response. This value is normally set to the media
+            type provided when a :class:`falcon.API` is initialized; however,
+            if created independently, this will default to the
+            ``DEFAULT_MEDIA_TYPE`` specified by Falcon.
+
+        media_handlers (Handlers): A dict-like object that allows you to
+            configure the media-types that you would like to handle.
+            By default, a handler is provided for the ``application/json``
+            media type.
     """
     __slots__ = (
         'keep_blank_qs_values',
         'auto_parse_form_urlencoded',
         'auto_parse_qs_csv',
         'strip_url_path_trailing_slash',
+        'default_media_type',
+        'media_handlers',
     )
 
     def __init__(self):
@@ -1400,3 +1701,38 @@ class RequestOptions(object):
         self.auto_parse_form_urlencoded = False
         self.auto_parse_qs_csv = True
         self.strip_url_path_trailing_slash = True
+        self.default_media_type = DEFAULT_MEDIA_TYPE
+        self.media_handlers = Handlers()
+
+
+class Forwarded(object):
+    """Represents a parsed Forwarded header.
+
+    (See also: RFC 7239, Section 4)
+
+    Attributes:
+        src (str): The value of the "for" parameter, or
+            ``None`` if the parameter is absent. Identifies the
+            node making the request to the proxy.
+        dest (str): The value of the "by" parameter, or
+            ``None`` if the parameter is absent. Identifies the
+            client-facing interface of the proxy.
+        host (str): The value of the "host" parameter, or
+            ``None`` if the parameter is absent. Provides the host
+            request header field as received by the proxy.
+        scheme (str): The value of the "proto" parameter, or
+            ``None`` if the parameter is absent. Indicates the
+            protocol that was used to make the request to
+            the proxy.
+    """
+
+    # NOTE(kgriffs): Use "client" since "for" is a keyword, and
+    # "scheme" instead of "proto" to be consistent with the
+    # falcon.Request interface.
+    __slots__ = ('src', 'dest', 'host', 'scheme')
+
+    def __init__(self):
+        self.src = None
+        self.dest = None
+        self.host = None
+        self.scheme = None
