@@ -22,6 +22,8 @@ from six import string_types as STRING_TYPES
 # See issue https://github.com/falconry/falcon/issues/556
 from six.moves import http_cookies
 
+from falcon import DEFAULT_MEDIA_TYPE
+from falcon.media import Handlers
 from falcon.response_helpers import (
     format_header_value_list,
     format_range,
@@ -61,10 +63,18 @@ class Response(object):
                 codes. They all start with the ``HTTP_`` prefix, as in:
                 ``falcon.HTTP_204``.
 
-        body (str or unicode): String representing response content. If
-            Unicode, Falcon will encode as UTF-8 in the response. If
-            data is already a byte string, use the data attribute
-            instead (it's faster).
+        media (object): A serializable object supported by the media handlers
+            configured via :class:`falcon.RequestOptions`.
+
+            See :ref:`media` for more information regarding media handling.
+
+        body (str or unicode): String representing response content.
+
+            If set to a Unicode type (``unicode`` in Python 2, or
+            ``str`` in Python 3), Falcon will encode the text as UTF-8
+            in the response. If the content is already a byte string,
+            use the :attr:`data` attribute instead (it's faster).
+
         data (bytes): Byte string representing response content.
 
             Use this attribute in lieu of `body` when your content is
@@ -97,6 +107,7 @@ class Response(object):
             Content-Length header to the WSGI server. Consequently, the
             server may choose to use chunked encoding or one of the
             other strategies suggested by PEP-3333.
+
         context (dict): Dictionary to hold any data about the response which is
             specific to your app. Falcon itself will not interact with this
             attribute after it has been initialized.
@@ -130,28 +141,45 @@ class Response(object):
     )
 
     # Child classes may override this
-    context_type = None
+    context_type = dict
 
     def __init__(self, options=None):
         self.status = '200 OK'
         self._headers = {}
 
-        self.options = ResponseOptions() if options is None else options
+        self.options = options if options else ResponseOptions()
 
         # NOTE(tbug): will be set to a SimpleCookie object
         # when cookie is set via set_cookie
         self._cookies = None
+        self._media = None
 
         self.body = None
         self.data = None
         self.stream = None
         self.stream_len = None
 
-        if self.context_type is None:
-            # PERF(kgriffs): The literal syntax is more efficient than dict().
-            self.context = {}
-        else:
-            self.context = self.context_type()
+        self.context = self.context_type()
+
+    @property
+    def media(self):
+        return self._media
+
+    @media.setter
+    def media(self, obj):
+        self._media = obj
+
+        if not self.content_type:
+            self.content_type = self.options.default_media_type
+
+        handler = self.options.media_handlers.find_by_media_type(
+            self.content_type,
+            self.options.default_media_type
+        )
+        self.data = handler.serialize(self._media)
+
+    def __repr__(self):
+        return '<%s: %s>' % (self.__class__.__name__, self.status)
 
     def set_stream(self, stream, stream_len):
         """Convenience method for setting both `stream` and `stream_len`.
@@ -481,7 +509,7 @@ class Response(object):
                  anchor=None, hreflang=None, type_hint=None):
         """Add a link header to the response.
 
-        See also: https://tools.ietf.org/html/rfc5988
+        (See also: RFC 5988, Section 1)
 
         Note:
             Calling this method repeatedly will cause each link to be
@@ -496,8 +524,9 @@ class Response(object):
                 link. Will be converted to a URI, if necessary, per
                 RFC 3987, Section 3.1.
             rel (str): Relation type of the link, such as "next" or
-                "bookmark". See also http://goo.gl/618GHr for a list
-                of registered link relation types.
+                "bookmark".
+
+                (See also: http://www.iana.org/assignments/link-relations/link-relations.xhtml)
 
         Keyword Args:
             title (str): Human-readable label for the destination of
@@ -633,13 +662,22 @@ class Response(object):
             case, raising ``falcon.HTTPRangeNotSatisfiable`` will do the right
             thing.
 
-            See also: http://goo.gl/Iglhp
+        (See also: RFC 7233, Section 4.2)
         """,
         format_range)
 
     content_type = header_property(
         'Content-Type',
-        'Set the Content-Type header.')
+        """Sets the Content-Type header.
+
+        The ``falcon`` module provides a number of constants for
+        common media types, including ``falcon.MEDIA_JSON``,
+        ``falcon.MEDIA_MSGPACK``, ``falcon.MEDIA_YAML``,
+        ``falcon.MEDIA_XML``, ``falcon.MEDIA_HTML``,
+        ``falcon.MEDIA_JS``, ``falcon.MEDIA_TEXT``,
+        ``falcon.MEDIA_JPEG``, ``falcon.MEDIA_PNG``,
+        and ``falcon.MEDIA_GIF``.
+        """)
 
     etag = header_property(
         'ETag',
@@ -678,17 +716,17 @@ class Response(object):
         """Value to use for the Vary header.
 
         Set this property to an iterable of header names. For a single
-        asterisk or field value, simply pass a single-element ``list`` or
-        ``tuple``.
+        asterisk or field value, simply pass a single-element ``list``
+        or ``tuple``.
 
-        "Tells downstream proxies how to match future request headers
-        to decide whether the cached response can be used rather than
-        requesting a fresh one from the origin server."
-
-        (Wikipedia)
-
-        See also: http://goo.gl/NGHdL
+        The "Vary" header field in a response describes what parts of
+        a request message, aside from the method, Host header field,
+        and request target, might influence the origin server's
+        process for selecting and representing this response.  The
+        value consists of either a single asterisk ("*") or a list of
+        header field names (case-insensitive).
 
+        (See also: RFC 7231, Section 7.1.4)
         """,
         format_header_value_list)
 
@@ -710,6 +748,23 @@ class Response(object):
 
         """)
 
+    def _set_media_type(self, media_type=None):
+        """Wrapper around set_header to set a content-type.
+
+        Args:
+            media_type: Media type to use for the Content-Type
+                header.
+
+        """
+
+        # PERF(kgriffs): Using "in" like this is faster than using
+        # dict.setdefault (tested on py27).
+        set_content_type = (media_type is not None and
+                            'content-type' not in self._headers)
+
+        if set_content_type:
+            self.set_header('content-type', media_type)
+
     def _wsgi_headers(self, media_type=None, py2=PY2):
         """Convert headers into the format expected by WSGI servers.
 
@@ -720,14 +775,7 @@ class Response(object):
         """
 
         headers = self._headers
-
-        # PERF(kgriffs): Using "in" like this is faster than using
-        # dict.setdefault (tested on py27).
-        set_content_type = (media_type is not None and
-                            'content-type' not in headers)
-
-        if set_content_type:
-            headers['content-type'] = media_type
+        self._set_media_type(media_type)
 
         if py2:
             # PERF(kgriffs): Don't create an extra list object if
@@ -762,10 +810,25 @@ class ResponseOptions(object):
             default to ``False``. This can make testing easier by
             not requiring HTTPS. Note, however, that this setting can
             be overridden via `set_cookie()`'s `secure` kwarg.
+
+        default_media_type (str): The default media-type to use when
+            deserializing a response. This value is normally set to the media
+            type provided when a :class:`falcon.API` is initialized; however,
+            if created independently, this will default to the
+            ``DEFAULT_MEDIA_TYPE`` specified by Falcon.
+
+        media_handlers (Handlers): A dict-like object that allows you to
+            configure the media-types that you would like to handle.
+            By default, a handler is provided for the ``application/json``
+            media type.
     """
     __slots__ = (
         'secure_cookies_by_default',
+        'default_media_type',
+        'media_handlers',
     )
 
     def __init__(self):
         self.secure_cookies_by_default = True
+        self.default_media_type = DEFAULT_MEDIA_TYPE
+        self.media_handlers = Handlers()
