@@ -14,14 +14,15 @@
 
 """Default router and utility functions.
 
-This package implements Falcon's default routing engine, and also
-includes utility functions to aid in the implementation of custom
+This package implements Falcon's default routing engine, field converter
+classes, and utility functions to aid in the implementation of custom
 routers.
 """
 
-from falcon.routing.compiled import CompiledRouter
+from falcon.routing.compiled import CompiledRouter, CompiledRouterOptions  # NOQA
 from falcon.routing.util import create_http_method_map  # NOQA
 from falcon.routing.util import compile_uri_template  # NOQA
+from falcon.routing.converters import *  # NOQA
 
 
 DefaultRouter = CompiledRouter
