@@ -246,7 +246,7 @@ class Cookie(object):
 
 
 def simulate_request(app, method='GET', path='/', query_string=None,
-                     headers=None, body=None, file_wrapper=None,
+                     headers=None, body=None, file_wrapper=None, wsgierrors=None,
                      params=None, params_csv=True, protocol='http'):
         """Simulates a request to a WSGI application.
 
@@ -286,6 +286,8 @@ def simulate_request(app, method='GET', path='/', query_string=None,
                 environ (default: ``None``). This can be used to test
                 high-performance file transmission when `resp.stream` is
                 set to a file-like object.
+            wsgierrors (io): The stream to use as *wsgierrors*
+                (default ``sys.stderr``)
 
         Returns:
             :py:class:`~.Result`: The result of the request
@@ -323,6 +325,7 @@ def simulate_request(app, method='GET', path='/', query_string=None,
             headers=headers,
             body=body,
             file_wrapper=file_wrapper,
+            wsgierrors=wsgierrors,
         )
 
         srmock = StartResponseMock()
@@ -600,49 +603,49 @@ class TestClient(object):
     def simulate_get(self, path='/', **kwargs):
         """Simulates a GET request to a WSGI application.
 
-        See also: :py:meth:`falcon.testing.simulate_get`.
+        (See also: :py:meth:`falcon.testing.simulate_get`)
         """
         return simulate_get(self.app, path, **kwargs)
 
     def simulate_head(self, path='/', **kwargs):
         """Simulates a HEAD request to a WSGI application.
 
-        See also: :py:meth:`falcon.testing.simulate_head`.
+        (See also: :py:meth:`falcon.testing.simulate_head`)
         """
         return simulate_head(self.app, path, **kwargs)
 
     def simulate_post(self, path='/', **kwargs):
         """Simulates a POST request to a WSGI application.
 
-        See also: :py:meth:`falcon.testing.simulate_post`.
+        (See also: :py:meth:`falcon.testing.simulate_post`)
         """
         return simulate_post(self.app, path, **kwargs)
 
     def simulate_put(self, path='/', **kwargs):
         """Simulates a PUT request to a WSGI application.
 
-        See also: :py:meth:`falcon.testing.simulate_put`.
+        (See also: :py:meth:`falcon.testing.simulate_put`)
         """
         return simulate_put(self.app, path, **kwargs)
 
     def simulate_options(self, path='/', **kwargs):
         """Simulates an OPTIONS request to a WSGI application.
 
-        See also: :py:meth:`falcon.testing.simulate_options`.
+        (See also: :py:meth:`falcon.testing.simulate_options`)
         """
         return simulate_options(self.app, path, **kwargs)
 
     def simulate_patch(self, path='/', **kwargs):
         """Simulates a PATCH request to a WSGI application.
 
-        See also: :py:meth:`falcon.testing.simulate_patch`.
+        (See also: :py:meth:`falcon.testing.simulate_patch`)
         """
         return simulate_patch(self.app, path, **kwargs)
 
     def simulate_delete(self, path='/', **kwargs):
         """Simulates a DELETE request to a WSGI application.
 
-        See also: :py:meth:`falcon.testing.simulate_delete`.
+        (See also: :py:meth:`falcon.testing.simulate_delete`)
         """
         return simulate_delete(self.app, path, **kwargs)
 
